#include "img_tool.h"

#include <stdio.h>

#include "typedfs.h"


/** R1 := R1 or R2 */
void OrR(void *R1, const void *R2, unsigned maxX)
{
  while(maxX>=4)
  {
    *(uint32_t*)R1 |= *(uint32_t*)R2;
    R1 = (uint32_t*)R1 + 1;
    R2 = (uint32_t*)R2 + 1;
    maxX -= 4;
  }
  while(maxX>=1)
  {
    *(int8_t*)R1 |= *(int8_t*)R2;
    R1 = (int8_t*)R1 + 1;
    R2 = (int8_t*)R2 + 1;
    maxX--;
  }
}


/** R1 := R1 and R2 */
void AndR(void *R1, const void *R2, unsigned maxX)
{
  while(maxX>=4)
  {
    *(uint32_t*)R1 &= *(uint32_t*)R2;
    R1 = (uint32_t*)R1 + 1;
    R2 = (uint32_t*)R2 + 1;
    maxX -= 4;
  }
  while(maxX>=1)
  {
    *(int8_t*)R1 &= *(int8_t*)R2;
    R1 = (int8_t*)R1 + 1;
    R2 = (int8_t*)R2 + 1;
    maxX--;
  }
}


/** In place right shift R1:=R1 shr 1 */
void ShrR(void *R, unsigned maxX)
{
uint8_t PrevVal, val;
  if(R==NULL || maxX==0) return;

  PrevVal = (*(uint8_t*)R & 0x80);
  while(maxX-->0)
  {
    val = *(uint8_t*)R;
    *(uint8_t*)R = PrevVal | (val>>1);
    R = (uint8_t*)R + 1;
    PrevVal = (val & 1) ? 0x80 : 0;
  }
}


/** In place left shift R1:=R1 shl 1 */
void ShlR(void *R, unsigned maxX, unsigned char LastBit)
{
uint8_t PrevVal, val;
  if(R==NULL || maxX==0) return;

  R = (int8_t*)R + maxX - 1;

  PrevVal = *(uint8_t*)R & LastBit;
  while(maxX-->0)
  {
    val = *(uint8_t*)R;
   *(int8_t*)R = PrevVal | (val<<1);
   R = (uint8_t*)R - 1;
   PrevVal = (val & 0x80) ? 1 : 0;
  }
}


/** Find a maximum from two BYTE vectors: R1 = max(R1,R2).
 * @param[in,out] R1	First vector and result on return.
 * @param[in]	  R2	Second vector.
 * @param[in]	  size	Amount of bytes. */
void MaxU8(unsigned char *R1, const unsigned char *R2, unsigned size)
{
  while(size-->0)
  {
    if(*R2 > *R1) *R1=*R2;
    R1++;
    R2++;
  }
}


/** Find a minumum from two BYTE vectors: R1 = min(R1,R2).
 * @param[in,out] R1	First vector and result on return.
 * @param[in]	  R2	Second vector.
 * @param[in]	  size	Amount of bytes. */
void MinU8(unsigned char *R1, const unsigned char *R2, unsigned size)
{
  while(size-->0)
  {
    if(*R2 < *R1) *R1=*R2;
    R1++;
    R2++;
  }
}


void MaxU16(unsigned char *R1, const unsigned char *R2, unsigned size)
{
  while(size-->0)
  {
    if(*(uint16_t*)R2 > *(uint16_t*)R1) *(uint16_t*)R1=*(uint16_t*)R2;
    R1+=2;
    R2+=2;
  }
}


void MinU16(unsigned char *R1, const unsigned char *R2, unsigned size)
{
  while(size-->0)
  {
    if(*(uint16_t*)R2 < *(uint16_t*)R1) *(uint16_t*)R1=*(uint16_t*)R2;
    R1+=2;
    R2+=2;
  }
}
